<?php
/**
 * PeachPay Product API
 *
 * @package PeachPay
 */

if ( ! defined( 'PEACHPAY_ABSPATH' ) ) {
	exit;
}

/**
 * Gets the parent id of a product. If the product does not have a parent then it returns its own Id.
 *
 * @param int $id The variation or child product id.
 * @return int The id of the parent or the id of the product you are checking.
 */
function peachpay_product_parent_id( $id ) {
	$product = wc_get_product( $id );

	if ( ! $product ) {
		return $id;
	}

	if ( $product instanceof WC_Product_Variation ) {
		return $product->get_parent_id();
	}

	return $id;
}

/**
 * Gets the product price.
 *
 * @param WC_Product $wc_product The product to get the price for.
 */
function peachpay_product_price( WC_Product $wc_product ) {
	$price = wc_get_price_excluding_tax( $wc_product );

	return (float) apply_filters( 'peachpay_product_price', $price, $wc_product );
}

/**
 * Allows retrieval of the WC_Product out of a WC Line item array with strong typing
 *
 * @param array $wc_line_item The Woocommerce Line item.
 * @return WC_Product
 */
function peachpay_product_from_line_item( $wc_line_item ) {
	return $wc_line_item['data'];
}

/**
 * Get the price product price to render in the modal.
 *
 * @param WC_Product $wc_product The product to get the display price for.
 */
function peachpay_product_display_price( WC_Product $wc_product ) {
	$price = 0;
	if ( 'incl' === WC()->cart->get_tax_price_display_mode() ) {
		$price = wc_get_price_including_tax( $wc_product );
	} else {
		$price = wc_get_price_excluding_tax( $wc_product );
	}

	return (float) apply_filters( 'peachpay_product_display_price', $price, $wc_product, ( 'incl' === WC()->cart->get_tax_price_display_mode() ) );
}

/**
 * Gets the products sin stock status.
 *
 * @param int $product_id The id of the product or variation.
 */
function peachpay_product_stock_status( int $product_id ) {
	$product = wc_get_product( $product_id );
	if ( ! $product ) {
		return 'outofstock';
	}

	/**
	 * Filters stock status for non wc inventory plugin support.
	 *
	 * @param string $stock_status The status of the stock
	 * @param int    $id           The id of the given product or variation
	 */
	return apply_filters( 'peachpay_product_stock_status', $product->get_stock_status(), $product_id );
}

/**
 * Takes in a product ID and if that product has variations will return an array
 * of all the product's variations. If no variations exist it returns an empty
 * string.
 *
 * @param int $id the product Id.
 * @return array of variation attributes including both the label and value.
 */
function peachpay_product_variation_attributes( $id ) {
	$variations = '';
	$product    = wc_get_product( $id );
	if ( ! $product ) {
		return '';
	}
	if ( $product instanceof WC_Product_Variation ) {
		$variations = $product->get_variation_attributes( true );
	}
	return $variations;
}

/**
 * Gets the formatted product variation name.
 *
 * @param int $id The product variation id.
 */
function peachpay_product_variation_name( $id ) {
	$product = wc_get_product( $id );
	if ( ! $product ) {
		return '';
	}
	return wc_get_formatted_variation( $product, true, false );
}

/**
 * Retrieve the given product's image for use in the checkout window.
 *
 * @param WC_Product $product The product to get image data for.
 * @return array|false Image data for the product, or false it it doesn't exist
 * or the option is turned off.
 */
function peachpay_product_image( WC_Product $product ) {
	if ( peachpay_get_settings_option( 'peachpay_general_options', 'display_product_images' ) ) {
		return wp_get_attachment_image_src( $product->get_image_id() );
	}
}

/**
 * Retrieve the given product's upsell/cross-sell item data for use in the checkout window.
 *
 * @param WC_Product $product The product to get upsell or cross-sell item's data.
 * @param string     $type Type of linked product i.e. upsell, cross-sell.
 * @return array|false Upsell/cross-sell item data for the product, or false it it doesn't
 * exist or the option is turned off.
 */
function peachpay_get_linked_products( WC_Product $product, $type ) {
	if ( 'upsell' === $type && ! peachpay_get_settings_option( 'peachpay_related_products_options', 'display_woocommerce_products_upsell' ) || 'cross' === $type && ! peachpay_get_settings_option( 'peachpay_related_products_options', 'display_woocommerce_products_cross_sell' ) ) {
		return false;
	}
	$linked_products_ids = 'upsell' === $type ? $product->get_upsell_ids() : $product->get_cross_sell_ids();
	if ( ! $linked_products_ids ) {
		return false;
	}
	$linked_product_items = array();
	foreach ( $linked_products_ids as $id ) {
		$linked_product = wc_get_product( $id );
		if ( $linked_product && ( $linked_product->get_stock_status() === 'instock' || $linked_product->get_stock_status() === 'onbackorder' ) ) {
			$item = array(
				'id'         => $linked_product->get_id(),
				'name'       => $linked_product->get_name(),
				'price'      => $linked_product->get_price_html(),
				'variable'   => $linked_product->is_type( 'variable' ),
				'bundle'     => $linked_product->is_type( 'bundle' ) ? peachpay_is_variation_bundle( $linked_product ) : false,
				'img_src'    => is_array( peachpay_product_image( $linked_product ) ) ? peachpay_product_image( $linked_product )[0] : '',
				'permalink'  => get_permalink( $id ),
				'attributes' => $linked_product->is_type( 'variable' ) ? peachpay_get_attribute_data( $linked_product ) : false,
				'sale'       => $linked_product->is_on_sale(),
			);
			array_push( $linked_product_items, $item );
		}
	}
	return $linked_product_items;
}

/**
 * Check if bundle product contains variation product
 *
 * @param WC_Product_Bundle $bundle The bundle product to get product data from.
 */
function peachpay_is_variation_bundle( WC_Product_Bundle $bundle ) {
	$bundle_products = $bundle->get_bundled_items();
	foreach ( $bundle_products as $bundle_product ) {
		$product = $bundle_product->get_product();
		if ( $product->is_type( 'variable' ) ) {
			return true;
		}
	}
	return false;
}

/**
 * Get variable product's attribute data.
 *
 * @param WP_Product $product Variation product.
 */
function peachpay_get_attribute_data( $product ) {
	$attributes = $product->get_attributes();
	$attr_data  = array();
	foreach ( $attributes as $attr ) {
		$data = array(
			'label'   => wc_attribute_label( $attr['name'] ),
			'name'    => sanitize_title( $attr['name'] ),
			'options' => $attr->get_slugs(),
		);

		array_push( $attr_data, $data );
	}
	return $attr_data;
}
